import threading
import socket

encoding = 'utf-8'
BUFSIZE = 1024


# a read thread, read data from remote
class Reader(threading.Thread):
    def __init__(self, client):
        threading.Thread.__init__(self)
        self.client = client

    def run(self):
        while True:
            data = self.client.recv(BUFSIZE)
            if (data):
                string = bytes.decode(data, encoding)
                print(string, end='')
            else:
                break
        print("close:", self.client.getpeername())

    def readline(self):
        rec = self.inputs.readline()
        if rec:
            string = bytes.decode(rec, encoding)
            if len(string) > 2:
                string = string[0:-2]
            else:
                string = ' '
        else:
            string = False
        return string


# a listen thread, listen remote connect
# when a remote machine request to connect, it will create a read thread to handle
class Listener(threading.Thread):
    def __init__(self, port):
        threading.Thread.__init__(self)
        self.port = port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind(("127.0.0.1", port))
        self.sock.listen(0)

    def run(self):
        print("listener started")
        while True:
            client, cltadd = self.sock.accept()
            Reader(client).start()
            cltadd = cltadd
            print("accept a connect")


lst = Listener(9011)  # create a listen thread
lst.start()  # then start



import sys

sys.path.append('')

from elf_py_utils import BaseUtil, RedisUtil

logger = BaseUtil.set_logging('RedisCMD')

redis = RedisUtil.connect_redis()


def redis_get(param: str):
    key = param.split(' ')[1]
    result = redis.get(key)
    if result is None:
        result = 'nil'
    print(result)


def redis_set(param: str):
    key = param.split(' ')[1]
    value = param.split(' ')[2]
    redis.set(key, value)
    print(1)


def redis_hget(param: str):
    key = param.split(' ')[1]
    filed = param.split(' ')[2]
    result = redis.hget(key, filed)
    if result is None:
        result = 'nil'
    print(result)


def redis_hgetall(param: str):
    key = param.split(' ')[1]
    result = redis.hgetall(key)
    if result is None:
        result = 'nil'
    print(result)


cmd_dict = {
    'get': redis_get,
    'set': redis_set,
    'hget': redis_hget,
    'hgetall': redis_hgetall
}

"""
del
hgetset


"""

while True:
    cmd = str(input('> '))
    cmd_type = cmd.split(' ')[0]
    if cmd == 'quit':
        break
    elif cmd_type in cmd_dict:
        cmd_param = cmd[len(cmd_type):]
        func = cmd_dict[cmd_type]
        func(cmd_param)
    else:
        print("Can't resolve command like %s" % cmd)



# -*- coding: UTF-8 -*-
import redis
from redis.sentinel import Sentinel
import rediscluster

from elf_py_utils import BaseUtil

logger = BaseUtil.set_logging('RedisUtil')
config = BaseUtil.get_config()


def connect_redis_single():
    """连接Redis-单节点

    配置示例（yaml格式）：
    redis:
      type: sentinel
      single:
        host_port: redis

    :return: redis_client
    """
    host_port = config.read_config('redis.single.host_port')
    host = host_port.split(':')[0]
    port = host_port.split(':')[1]
    pool = redis.ConnectionPool(host=host,
                                port=port,
                                decode_responses=True)
    redis_client = redis.Redis(connection_pool=pool, charset='utf-8', encoding='utf-8')
    logger.info("Connect to Redis success")
    return redis_client


# 连接集群Redis
def connect_redis_cluster() -> rediscluster.client:
    """连接Redis-集群

    配置示例（yaml格式）：

    redis:
      type: cluster
      cluster:
        host_port: 127.0.0.1:6379,127.0.0.2:6379,127.0.0.3:6379

    :return: redis_client
    """
    host_port_list = config.read_config('redis.cluster.host_port').split(',')
    startup_nodes = []
    for host_port in host_port_list:
        host = host_port.split(':')[0]
        port = host_port.split(':')[1]
        startup_nodes.append({'host': host, 'port': port})
    redis_client = rediscluster.RedisCluster(startup_nodes=startup_nodes,
                                             decode_responses=True,
                                             skip_full_coverage_check=True)
    logger.info("Connect to Redis Cluster success")
    return redis_client


# 连接哨兵Redis
def connect_redis_sentinel() -> redis.client:
    """连接Redis-哨兵

    配置示例（yaml格式）：
    redis:
      type: sentinel
      sentinel:
        db: 1
        password: 7crplVt9TnunDrMY7tu
        service_name: mymaster
        host_port: 10.147.243.20:26379,10.147.243.21:26379,10.147.243.22:26379

    :return: redis_client
    """
    host_port_list = config.read_config('redis.sentinel.host_port').split(',')
    db = int(config.read_config('redis.sentinel.db'))
    password = str(config.read_config('redis.sentinel.password'))
    service_name = str(config.read_config('redis.sentinel.service_name'))

    sentinel_address = []
    for host_port in host_port_list:
        sentinel_address.append((host_port.split(':')[0], int(host_port.split(':')[1])))

    my_sentinel = Sentinel(sentinel_address, socket_timeout=2000)
    master = my_sentinel.master_for(service_name, socket_timeout=2000, db=db, password=password, encoding='utf-8')
    slave = my_sentinel.slave_for(service_name, socket_timeout=2000, db=db, password=password, encoding='utf-8')

    return master


connect_redis_type_dict = {
    'single': connect_redis_single,
    'cluster': connect_redis_cluster,
    'sentinel': connect_redis_sentinel

}

redis_is_cluster_dict = {'true': connect_redis_cluster,
                         'false': connect_redis_single}


def connect_redis() -> redis.client:
    """连接Redis
    根据配置文件获取Redis的部署模式，
    并根据是否为集群模式来执行不同的连接过程

    :return: redis_client
    """
    redis_type = config.read_config('redis.type')
    logger.info('Redis Type: {}'.format(redis_type))

    redis_client = connect_redis_type_dict[redis_type]()

    if redis_client is None:
        logger.error('连接失败，请检查配置信息')

    return redis_client



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['elf_py_utils',
 'elf_py_utils.CodeGen',
 'elf_py_utils.constants',
 'elf_py_utils.utils']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'elf-py-utils',
    'version': '0.1.3.3',
    'description': '',
    'long_description': '# elf-py-utils项目说明\n\n> 小精灵Python工具包\n\n## 前言\n\n- 目标使用者：从事运维工程师、后端开发工程师等相关岗位，并熟悉python基础语法规则\n- 本包的工作：在前人的基础上，对当前已有的工具包进行二次封装，并提供一个统一的配置文件来进行配置管理，意图提供一个类似于Spring Boot的框架方便使用\n\n## 当前功能\n\n### Redis工具类\n\n#### 配置示例\n\n通过配置文件，实现一种代码连接多种Redis部署方式\n\n```yaml\nredis:\n  # 需要在同一命名空间中定义redis的服务发现\n  # redis部署方式 单节点-single 集群模式-cluster 哨兵模式-sentinel\n  type: cluster\n  single:\n    host_port: redis\n  cluster:\n    host_port: redis-cluster:6379\n  sentinel:\n    db: 1\n    password: password\n    service_name: mymaster\n    host_port: redis-sentinel-01:6379,redis-sentinel-02:6379,redis-sentinel-03:6379\n```\n\n### 调用\n\n```python\nfrom elf_py_utils import RedisUtil\n\nredis = RedisUtil.connect_redis()\n```\n\n### 依赖\n\n```txt\nredis==3.5.3\nredis-py-cluster==2.1.3\n```\n\n\n',
    'author': 'TreeOfWord',
    'author_email': 'li_163jx@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# !/usr/bin/python
# -*- coding: UTF-8 -*-
import configparser
import logging
import time
import os
import openpyxl

from elf_py_utils.ConfigUtil import ConfigReader


def get_config() -> ConfigReader:
    """
    获取配置信息类

    :return: 配置信息类
    """
    bootstrap = get_bootstrap()
    config_file_name = str(bootstrap.read_config('profiles.config.name'))
    config_file_path = str(bootstrap.read_config('profiles.config.path'))
    config_file_type = config_file_name.split('.')[1]
    config_path_yaml = os.path.join(config_file_path)
    config = ConfigReader(config_path_yaml, config_file_type)

    return config


def get_bootstrap():
    """
    获取bootstrap信息类

    :return: bootstrap配置信息类
    """
    bootstrap = ConfigReader('bootstrap.yaml', 'yaml')
    return bootstrap


def load_config(section: str) -> dict:
    """
    加载配置文件，配置文件的相对地址已写死

    :param section: 配置节名
    :return: section节下的配置字典
    """
    # 相对于顶级文件的地址
    file = "common/config/local.ini"  # 这种写法可太不好了
    # 创建配置文件对象
    con = configparser.ConfigParser()

    # 读取文件
    con.read(file, encoding="utf-8")

    # 获取特定section
    items = con.items(section, raw=True)  # 返回结果为元组

    # 可以通过dict方法转换为字典
    items = dict(items)
    return items


# 配置日志设置
def set_logging(name='Main'):
    log_formatter = logging.Formatter(get_config().read_config('logging.formatter'))
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)

    console_handler = logging.StreamHandler()
    console_handler.setFormatter(log_formatter)
    logger.addHandler(console_handler)

    return logger


logger = set_logging('BaseUtil')


def read_excel(title_row: int, file_name: str) -> list[dict]:
    """
    读取excel文件，返回一列表，列表中元素为字典

    :param title_row: 列名所在的行索引
    :param file_name: .xlsx文件地址
    :return: excel文件内容，列表中每个元素即为表格中的一行数据
    """
    start_time = time.time()
    excel_content = []
    # 加载excel
    wb = openpyxl.load_workbook(file_name)
    ws = wb.active
    # 获取excel表格的列名
    column_name = []
    for y in range(1, ws.max_column + 1):
        column_name.append(ws.cell(title_row, y).value)
    for x in range(title_row + 1, ws.max_row + 1):
        record = {}
        for y in range(1, ws.max_column + 1):
            record[column_name[y - 1]] = ws.cell(x, y).value
        excel_content.append(record)
        if x % 500 == 0:
            logger.info('[读取excel]总行数：%s，当前行数：%s，进度：%s%s，当前耗时：%s秒',
                        ws.max_row, x, round(x / ws.max_row * 100, 2), '%', round(time.time() - start_time))
    return excel_content


def read_csv(title_row: int, file_name: str) -> list[dict]:
    """
    读取csv文件，返回一列表，列表中元素为字典

    :param title_row:
    :param file_name:
    :return: list[dict]
    """
    logger.info('正在读取csv文件')
    start_time = time.time()
    csv_content = []
    # 加载csv文件
    csv_file = open(file_name, 'r', encoding='utf-8')
    # 获取csv文件的列名
    column_name = csv_file.readline().replace('\n', '').split(',')
    n = 1
    while True:
        n += 1
        row = csv_file.readline()
        if row:
            record = {}
            row_list = row.replace('\n', '').split(',')
            if len(row_list) == len(column_name):
                if n % 500000 == 0:
                    logger.info('当前行数为：%d', n)
                for i in range(len(row_list)):
                    record[column_name[i]] = row_list[i]
                csv_content.append(record)
            else:
                logger.info('csv文件的第%d行数据格式有误', n)
                logger.info('其内容为：%s', row)
                break
        else:
            logger.info('csv文件读取完毕')
            break
    end_time = time.time()
    logger.info('处理csv文件耗时为：%.3f秒', (end_time - start_time))
    return csv_content


def read_folder(folder_path: str, suffix: str = '') -> list:
    """读取文件夹，获取文件夹中所有文件名列表并返回

    * 返回结果仅含有.suffix文件

    :param folder_path: 文件夹路径
    :param suffix: 文件后缀
    :return: list
    """
    files = os.listdir(folder_path)
    i = 0
    while i < len(files):
        if suffix != '':
            if suffix != files[i][-len(suffix):]:
                files.pop(i)
            else:
                i += 1
        else:
            i += 1
    return files



# !/usr/bin/python
# -*- coding: utf-8 -*-
import json


def load_json() -> dict:
    json_file = open('EntityInfo.json', 'r', encoding='utf-8')
    json_str = ''
    for line in json_file.readlines():
        json_str += line
    json_str.replace('\n', '')
    json_str.replace('\t', '')
    json_str.replace(' ', '')

    json_dict: dict = json.loads(json_str)
    return json_dict


def generate_py(dict_info: dict):
    entity_info = dict_info.get('entity_info')
    column_info = dict_info.get('column_info')
    # 1.文件头部内容
    text = "# !/usr/bin/python\n# -*- coding: utf-8 -*-\n"
    text += "from common.BaseEntity import BaseEntity\n\n"
    text += "column_name_list = ['id', 'create_time', 'create_by', 'update_time', 'update_by', 'remark'\n"
    for item in column_info.keys():
        text += ", '%s'" % item
    text += "]\n\n\n"
    # 2.类内容
    # 2.1 字段属性初始化
    text += "class %s(BaseEntity):\n" % entity_info
    for item in column_info.keys():
        column = "\t%s: %s = %s\n"
        if column_info.get(item).get('type') == 'str':
            column = column % (item, 'str', "''")
        elif column_info.get(item).get('type') == 'int':
            column = column % (item, 'int', 0)
        text += column
    # 2.2 类方法-__set方法
    for item in column_info.keys():
        line = "\n\tdef __set_%s(self, value: %s):\n\t\tself.%s = value\n"
        line = line % (item, column_info.get(item).get('type'), item)
        text += line
    text += '\n'
    # 2.3 类方法-通用get方法
    text += "\t# 通用get方法\n"
    text += "\tdef __get_id(self) -> str:\n\t\treturn self._get_sql_value(self.id)\n\n" \
            "\tdef __get_create_time(self) -> str:\n\t\treturn self._get_sql_value(self.create_time)\n\n" \
            "\tdef __get_create_by(self) -> str:\n\t\treturn self._get_sql_value(self.create_by)\n\n" \
            "\tdef __get_update_time(self) -> str:\n\t\treturn self._get_sql_value(self.update_time)\n\n" \
            "\tdef __get_update_by(self) -> str:\n\t\treturn self._get_sql_value(self.update_by)\n\n" \
            "\tdef __get_remark(self) -> str:\n\t\treturn self._get_sql_value(self.remark)\n\n"
    # 2.4 特有get方法
    text += "\t# 特有get方法-private-获取sql语句所需的value\n"
    for item in column_info.keys():
        line = "\tdef __get_%s(self) -> %s:\n\t\treturn self._get_sql_value(self.%s)\n\n"
        line = line % (item, column_info.get(item).get('type'), item)
        text += line
    # 2.5 set_value方法
    text += "\tdef set_value(self, value, value_name):\n\t\t" \
            "fun = self.value_name_map.get(value_name)\n\t\t" \
            "fun(self, value)\n\n"
    # 2.6 get_value方法
    text += "\tdef get_value(self, value_name: str):\n\t\t" \
            "value_name_get_dict = {\n"
    for item in column_info.keys():
        line = "\t\t\t'%s': self.%s,\n"
        line = line % (column_info.get(item).get('comment'), item)
        text += line
    text = text[:-2]
    text += "\n\t\t}\n"
    text += "\t\treturn value_name_get_dict.get(value_name)\n"
    # 2.7 generate_insert_sql生成insert的sql
    text += '\n'
    text += "\tdef generate_insert_sql(self) -> str:\n" \
            "\t\tsql = '(%s)'\n" \
            "\t\tvalue = ''\n" \
            "\t\tfor column_name in column_name_list:\n" \
            "\t\t\tvalue += str(self.value_name_get_dict[column_name](self))\n" \
            "\t\t\tvalue += ','\n" \
            "\t\tvalue = value[:-1]\n" \
            "\n" \
            "\t\treturn sql % value\n"
    # 2.8 value_name_map
    text += '\n'
    text += "\tvalue_name_map = {\n"
    for item in column_info.keys():
        line = "\t\t'%s': __set_%s,\n"
        line = line % (column_info.get(item).get('comment'), item)
        text += line
    text += '\t}\n\n'
    # 2.9 value_name_get_dict
    basic_column_list = ['id', 'create_time', 'create_by', 'update_time', 'update_by', 'remark']
    text += "\tvalue_name_get_dict = {\n"
    for item in column_info.keys():
        line = "\t\t'%s': __get_%s,\n"
        line = line % (item, item)
        text += line
    for item in basic_column_list:
        line = "\t\t'%s': __get_%s,\n"
        line = line % (item, item)
        text += line
    text += '\t}\n'

    out_file = open(entity_info + '.py', 'w', encoding='utf-8')
    out_file.writelines(text)
    out_file.close()


def main():
    json_dict = load_json()
    generate_py(json_dict)


if __name__ == '__main__':
    main()
